unit ARVFS;
//************************************************************//
//                         AND Report                        //
//  Eugene Hrulev (mrHru) Laxian Key, Omsk, Russia 1999,2001 //
//  Oleg Melnikov (muler) Laxian Key, Omsk, Russia 1999,2001 //
//************************************************************//

{$INCLUDE DEFINES.INC}

interface

uses
  Classes, SysUtils,
  {$IFDEF CLX}
    QDialogs, QMenus, QControls,
  {$ELSE}
    Dialogs, Menus, Controls,
  {$ENDIF}

  ARResStrings, ARUtils, ARSFSSetup;

type
  TVirtualFileSystem = class(TComponent)
  private
    FFilter         :string;
    FInitialDir     :string;
    FOptions        :TOpenOptions;
    FFiles          :TStringList;
    FOnExecuteSetup :TNotifyEvent;

  protected
    FTitle          :string;
    FWriteStream    :TStream;
    FReadStream     :TStream;
    FFileName       :string;
    FOptionsMenuItem:TMenuItem;
                
    procedure   SetOptionsMenuItem(AOptionsMenuItem:TMenuItem);virtual;
    procedure   SetParams(AParams:string);virtual;abstract;
    function    GetParams:string;virtual;abstract;
    function    GetMediaName:string;virtual;abstract;
    function    GetDisplayName:string;virtual;abstract;
  published
    property    Files:TStringList         read  FFiles;
    property    FileName:string           read  FFileName         write FFileName;
    property    Filter:string             read  FFilter           write FFilter;
    property    InitialDir:string         read  FInitialDir       write FInitialDir;
    property    Options:TOpenOptions      read  FOptions          write FOptions;
    property    OptionsMenuItem:TMenuItem read  FOptionsMenuItem  write SetOptionsMenuItem;
    property    Params:string             read  GetParams         write SetParams;

    property    DisplayName:string        read  GetDisplayName;
    property    MediaName:string          read  GetMediaName;

    function    Caption:string;virtual;

    function    CreateReadStream(AFileName:string):TStream;virtual;abstract;
    procedure   CloseReadStream;virtual;

    function    CreateWriteStream(AFileName:string):TStream;virtual;abstract;
    procedure   CloseWriteStream;virtual;

    function    OpenDialogExecute:boolean;virtual;abstract;
    function    SaveDialogExecute:boolean;virtual;abstract;

    //utilites
    procedure   CreateFile(AFileName:string);virtual;abstract;
    function    FileExists(AFileName:string):boolean;virtual;abstract;
    function    ExtractPath(AFileName:string):string;virtual;abstract;
    function    ExtractFileNameOnly(AFileName:string):string;virtual;abstract;

    procedure   ExecuteSetup(Sender: TObject);virtual;
    property    OnExecuteSetup:TNotifyEvent read FOnExecuteSetup      write FOnExecuteSetup;

    constructor Create(AOwner:TComponent);override;
    destructor  Destroy;override;
  end;

  TStandardFileSystem = class(TVirtualFileSystem)
  protected
    FDefaultExt :string;
    ExtList     :TAIStringList;
    property    OptionsMenuItem;
    function    GetDisplayName:string;override;
    function    GetMediaName:string;override;
    procedure   SetParams(AParams:string);override;
    function    GetParams:string;override;
  published
    property    DefaultExt:string           read FDefaultExt          write FDefaultExt;
    property    Title:string                read FTitle               write FTitle;

    procedure   ExecuteSetup(Sender: TObject);override;

    function    CreateReadStream(AFileName:string):TStream;override;
    function    CreateWriteStream(AFileName:string):TStream;override;

    function    OpenDialogExecute:boolean;override;
    function    SaveDialogExecute:boolean;override;

    procedure   CreateFile(AFileName:string);override;
    function    FileExists(AFileName:string):boolean;override;
    function    ExtractPath(AFileName:string):string;override;
    function    ExtractFileNameOnly(AFileName:string):string;override;

    constructor Create(AOwner:TComponent);override;
    destructor  Destroy;override;
  end;

procedure  Register;

implementation
{ $R *.res}

//////////////////////////   TVirtualFileSystem  /////////////////////
constructor TVirtualFileSystem.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);

  FFiles:=TStringList.Create;
end;

destructor  TVirtualFileSystem.Destroy;
begin
  FFiles.Free;
  inherited;
end;

procedure   TVirtualFileSystem.SetOptionsMenuItem(AOptionsMenuItem:TMenuItem);
begin
  FOptionsMenuItem:=AOptionsMenuItem;
end;

function    TVirtualFileSystem.Caption:string;
begin
  Result:=FileName;
end;

procedure   TVirtualFileSystem.CloseReadStream;
begin
  if Assigned(FReadStream) then FReadStream.Free;
end;

procedure   TVirtualFileSystem.CloseWriteStream;
begin
  if Assigned(FWriteStream) then FWriteStream.Free;
end;

procedure   TVirtualFileSystem.ExecuteSetup(Sender: TObject);
begin
  if Assigned(OnExecuteSetup) then OnExecuteSetup(Self);
end;

//////////////////////////   TStandardFileSystem  /////////////////////
constructor TStandardFileSystem.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  ExtList:=TAIStringList.Create('TStandardFileSystem.ExtList');
  ExtList.Duplicates:=dupIgnore;
  ExtList.Sorted    :=true;
end;

destructor TStandardFileSystem.Destroy;
begin
  ExtList.Free();

  inherited;
end;

function    TStandardFileSystem.OpenDialogExecute:boolean;
var OpenDialog:TOpenDialog;
begin
  OpenDialog:=TOpenDialog.Create(Self);
  try
    OpenDialog.FileName   :=FileName;
    OpenDialog.InitialDir :=InitialDir;
    OpenDialog.DefaultExt :=DefaultExt;
    OpenDialog.Filter     :=Filter;
    OpenDialog.Options    :=Options;
    OpenDialog.Title      :=Title;

    Result:=OpenDialog.Execute;
    FFiles.Assign(OpenDialog.Files);
    if not FileExists(OpenDialog.FileName) then CreateFile(OpenDialog.FileName);

    if Result then begin
      FileName:=OpenDialog.FileName;
    end;
  finally
    OpenDialog.Free;
  end;
end;

function    TStandardFileSystem.SaveDialogExecute:boolean;
var SaveDialog:TSaveDialog;
begin
  SaveDialog:=TSaveDialog.Create(Self);
  try
    SaveDialog.FileName   :=FileName;
    SaveDialog.InitialDir :=InitialDir;
    SaveDialog.DefaultExt :=DefaultExt;
    SaveDialog.Filter     :=Filter;
    SaveDialog.Options    :=Options;
    SaveDialog.Title      :=Title;

    Result:=SaveDialog.Execute;
    if Result then begin
      FileName:=SaveDialog.FileName;
    end;
  finally
    SaveDialog.Free;
  end;
end;

function    TStandardFileSystem.CreateReadStream(AFileName:string):TStream;
begin
  FFileName:=AFileName;

  FReadStream:=TFileStream.Create(FFileName, fmOpenRead or fmShareDenyNone);
  Result:=FReadStream;
end;

function    TStandardFileSystem.CreateWriteStream(AFileName:string):TStream;
begin
  FFileName:=AFileName;
  FWriteStream:=TFileStream.Create(AFileName, fmCreate or fmShareDenyWrite);
  Result:=FWriteStream;
end;

function    TStandardFileSystem.FileExists(AFileName:string):boolean;
begin
  Result:=SysUtils.FileExists(AFileName);
end;

procedure   TStandardFileSystem.CreateFile(AFileName:string);
var f:file;
begin
  AssignFile(f, AFileName);
  Rewrite(f);
  CloseFile(F);
end;

function    TStandardFileSystem.ExtractPath(AFileName:string):string;
begin
  Result:=ExtractFilePath(AFileName);
end;

function    TStandardFileSystem.ExtractFileNameOnly(AFileName:string):string;
var p:integer;
begin
  Result:=ExtractFileName(AFileName);
  p:=System.Pos('.', Result);
  if p > 0 then begin
    System.Delete(Result, p, Length(Result));
  end;
end;

function    TStandardFileSystem.GetDisplayName:string;
begin
  Result:=rsFS_StandardDisplayName;
end;

function    TStandardFileSystem.GetMediaName:string;
begin
  Result:=rsFS_StandardMediaName;
end;

procedure   TStandardFileSystem.ExecuteSetup(Sender: TObject);
var d:TARSFSSetupDlg;
begin
  inherited;

  d:=TARSFSSetupDlg.Create(Self);
  try
    d.ExtList.AddStrings(ExtList);
    d.Caption:=DisplayName;
    if d.ShowModal = mrOK then begin
      ExtList.AddStrings(d.ExtList);
    end;
  finally
    d.Free();
  end;
end;

function TStandardFileSystem.GetParams: string;
var sl:TStringList;
    i:integer;
begin
  sl:=TStringList.Create();
  try
    for i:=0 to ExtList.Count - 1 do begin
      sl.Add(ExtList[i]);
    end;

    Result:=sl.CommaText;
  finally
    sl.Free();
  end;
end;

procedure TStandardFileSystem.SetParams(AParams: string);
var i:integer;
    s:string;
begin
  ExtList.CommaText:=AParams;
  if ExtList.Count = 0 then begin
    ExtList.Add('AR');
    ExtList.Add('ART');
  end;

  Options:=Options + [ofOverwritePrompt];
  DefaultExt:='';

  if ExtList.Count > 0 then begin
    Filter:='';
    s:='';
    for i:=0 to ExtList.Count - 2 do begin
      s:=s + '*.' + ExtList[i] + ';';
    end;
    s:=s + '*.' + ExtList[ExtList.Count - 1];
    Filter:='Reports (' + s + ')|' + s + '|';

    DefaultExt:=ExtList[0];
  end;
  Filter:=Filter + 'All files (*.*)|*.*';
end;

////////////////////////////////////////////////////////////////
procedure Register;
begin
  RegisterComponents('ARComponents', [TStandardFileSystem]);
end;

end.




